import bpy

class MspGLMeshProperties(bpy.types.Panel):
	bl_idname = "MESH_PT_mspgl_properties"
	bl_label = "MspGL properties"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "data"

	@classmethod
	def poll(cls, context):
		return context.active_object.type=="MESH"

	def draw(self, context):
		mesh = context.active_object.data

		self.layout.prop(mesh, "winding_test")
		self.layout.prop(mesh, "smoothing")

		self.layout.separator()

		col = self.layout.column()
		col.label(text="Data selection")
		col.prop(mesh, "use_lines")
		col.prop(mesh, "vertex_groups")
		col.prop(mesh, "max_groups_per_vertex")

		self.layout.separator()

		col = self.layout.column()
		col.label(text="Texturing")
		col.prop(mesh, "use_uv")
		col.prop(mesh, "tbn_vecs")
		col.prop(mesh, "tbn_uvtex")

class MspGLObjectProperties(bpy.types.Panel):
	bl_idname = "OBJECT_PT_mspgl_properties"
	bl_label = "MspGL properties"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "object"

	def draw(self, context):
		obj = context.active_object

		self.layout.prop(obj, "compound")
		self.layout.prop(obj, "lod_for_parent")
		if obj.lod_for_parent:
			self.layout.prop(obj, "lod_index")

class MspGLMaterialProperties(bpy.types.Panel):
	bl_idname = "MATERIAL_PT_mspgl_properties"
	bl_label = "MspGL properties"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "material"

	@classmethod
	def poll(cls, context):
		return context.active_object.active_material is not None

	def draw(self, context):
		mat = context.active_object.active_material
		if not mat:
			return

		self.layout.prop(mat, "render_mode")
		if mat.render_mode=='CUSTOM':
			self.layout.prop(mat, "shader")
		elif mat.render_mode=='EXTERNAL':
			self.layout.prop(mat, "technique")
		self.layout.prop(mat, "array_atlas")
		if mat.array_atlas:
			self.layout.prop(mat, "array_layer")
		if mat.render_mode!='EXTERNAL':
			self.layout.prop(mat, "material_map")

class MspGLTextureNodeProperties(bpy.types.Panel):
	bl_idname = "NODE_PT_mspgl_properties"
	bl_label = "MspGL properties"
	bl_space_type = "NODE_EDITOR"
	bl_region_type = "UI"
	bl_category = "Item"

	@classmethod
	def poll(cls, context):
		node = context.active_node
		return node and node.type=='TEX_IMAGE'

	def draw(self, context):
		node = context.active_node
		if not node:
			return

		self.layout.prop(node, "default_filter")
		if not node.default_filter:
			self.layout.prop(node, "use_mipmap")
			self.layout.prop(node, "max_anisotropy")

classes = [MspGLMeshProperties, MspGLObjectProperties, MspGLMaterialProperties, MspGLTextureNodeProperties]

def register_properties():
	bpy.types.Mesh.winding_test = bpy.props.BoolProperty(name="Winding test", description="Perform winding test to skip back faces")
	bpy.types.Mesh.smoothing = bpy.props.EnumProperty(name="Smoothing", description="Smoothing method to use", default="MSPGL",
		items=(("NONE", "None", "No smoothing"),
			("BLENDER", "Blender", "Use Blender's vertex normals"),
			("MSPGL", "MspGL", "Compute vertex normals internally")))
	bpy.types.Mesh.use_lines = bpy.props.BoolProperty(name="Include lines", description="Include edges without faces as lines", default=False)
	bpy.types.Mesh.vertex_groups = bpy.props.BoolProperty(name="Vertex groups", description="Include vertex groups and weights", default=False)
	bpy.types.Mesh.max_groups_per_vertex = bpy.props.IntProperty(name="Max groups", description="Maximum amount of groups per vertex", min=1, max=4, default=2)
	bpy.types.Mesh.use_uv = bpy.props.EnumProperty(name="Use UV", description="Use UV coordinates", default="UNIT0",
		items=(("NONE", "None", "Ignore all UV coordinates"),
			("UNIT0", "Unit 0", "Use UV coordinates for unit 0"),
			("ALL", "All", "Use all UV coordinates")))
	bpy.types.Mesh.tbn_vecs = bpy.props.BoolProperty(name="TBN vectors", description="Compute tangent and binormal vectors for vertices", default=False)
	bpy.types.Mesh.tbn_uvtex = bpy.props.StringProperty(name="TBN UV layer", description="UV layer to use as basis for TBN vectors", default="")

	bpy.types.Object.compound = bpy.props.BoolProperty(name="Compound with parent", description="Join this object to its parent when exporting")
	bpy.types.Object.lod_for_parent = bpy.props.BoolProperty(name="LoD for parent", description="This object is a level of detail for its parent")
	bpy.types.Object.lod_index = bpy.props.IntProperty(name="LoD index", description="Index of the level of detail", min=1, max=16, default=1)

	bpy.types.Material.render_mode = bpy.props.EnumProperty(name="Render mode", description="How this material should be rendered", default="BUILTIN",
		items=(("BUILTIN", "Built-in", "Use built-in shaders"),
			("CUSTOM", "Custom shader", "Use a custom shader"),
			("EXTERNAL", "External technique", "Use an externally defined technique")))
	bpy.types.Material.technique = bpy.props.StringProperty(name="Custom technique", description="Name of an external technique to use for rendering")
	bpy.types.Material.shader = bpy.props.StringProperty(name="Custom shader", description="Name of an external technique to use for rendering")
	bpy.types.Material.array_atlas = bpy.props.BoolProperty(name="Texture array atlas", description="The material is stored in a texture array")
	bpy.types.Material.array_layer = bpy.props.IntProperty("Texture array layer", description="Layer of the texture array atlas to use")
	bpy.types.Material.material_map = bpy.props.BoolProperty(name="Material map", description="Make this material part of a material map")

	bpy.types.ShaderNodeTexImage.default_filter = bpy.props.BoolProperty(name="Default filter", description="Let the loading program determine filtering options")
	bpy.types.ShaderNodeTexImage.use_mipmap = bpy.props.BoolProperty(name="Use mipmaps", description="Use mipmaps (automatically generated) for the texture", default=True)
	bpy.types.ShaderNodeTexImage.max_anisotropy = bpy.props.FloatProperty(name="Maximum anisotropy", description="Maximum anisotropy to use in texture filtering", min=1, max=16, default=1)

	for c in classes:
		bpy.utils.register_class(c)

def unregister_properties():
	for c in classes:
		bpy.utils.unregister_class(c)
