#include <algorithm>
#include <msp/strings/format.h>
#include <msp/strings/lexicalcast.h>
#include <msp/strings/utils.h>
#include "error.h"
#include "misc.h"
#include "vertexformat.h"
#include <msp/gl/extensions/arb_vertex_shader.h>

using namespace std;

namespace Msp {
namespace GL {

VertexFormat::VertexFormat():
	count(0)
{ }

VertexFormat::VertexFormat(VertexAttribute a):
	count(1)
{
	attributes[0] = a;
}

VertexFormat VertexFormat::operator,(VertexAttribute a) const
{
	if(count>=MAX_ATTRIBUTES)
		throw invalid_operation("VertexFormat::operator,");

	VertexFormat r = *this;
	r.attributes[r.count++] = a;

	return r;
}

VertexFormat VertexFormat::operator,(unsigned i) const
{
	if(!count)
		throw invalid_operation("VertexFormat::operator,");

	VertexFormat r = *this;
	unsigned char &a = r.attributes[r.count-1];
	a = make_indexed_attribute(static_cast<VertexAttribute>(a), i);

	return r;
}

bool VertexFormat::operator==(const VertexFormat &other) const
{
	if(count!=other.count)
		return false;
	return equal(attributes, attributes+count, other.attributes);
}

unsigned VertexFormat::stride() const
{
	unsigned s = 0;
	for(const unsigned char *i=begin(); i!=end(); ++i)
		s += get_attribute_size(*i);
	return s;
}

int VertexFormat::offset(VertexAttribute attr) const
{
	unsigned sem = get_attribute_semantic(attr);
	unsigned sz = get_attribute_size(attr);
	unsigned offs = 0;
	for(const unsigned char *i=begin(); i!=end(); ++i)
	{
		if(get_attribute_semantic(*i)==sem)
		{
			if(get_attribute_size(*i)>=sz)
				return offs;
			else
				return -1;
		}
		else
			offs += get_attribute_size(*i);
	}

	return -1;
}


VertexAttribute make_indexed_attribute(VertexAttribute attr, unsigned index)
{
	unsigned base = attr;
	if(attr>=TEXCOORD1 && attr<=TEXCOORD4)
	{
		if(index>=4)
			throw out_of_range("make_indexed_attribute");
	}
	else if(attr>=RAW_ATTRIB1 && attr<=RAW_ATTRIB4)
		base &= 7;
	else if(attr<GENERIC1 || attr>GENERIC4)
		throw invalid_argument("make_indexed_attribute");

	static int max_attribs = -1;
	if(max_attribs<0)
		max_attribs = get_i(GL_MAX_VERTEX_ATTRIBS);

	if(static_cast<int>((base>>3)+index)>=max_attribs)
		throw out_of_range("make_indexed_attribute");

	return static_cast<VertexAttribute>(base+index*8);
}

void operator>>(const LexicalConverter &conv, VertexAttribute &a)
{
	const string &str = conv.get();
	if(str.size()==7 && !str.compare(0, 6, "VERTEX") && str[6]>='2' && str[6]<='4')
		a = static_cast<VertexAttribute>(VERTEX2+(str[6]-'2'));
	else if(str=="NORMAL3")
		a = NORMAL3;
	else if(str.size()==12 && !str.compare(0, 5, "COLOR") && str[5]>='3' && str[5]<='4' && !str.compare(6, 6, "_FLOAT"))
		a = static_cast<VertexAttribute>(COLOR3_FLOAT+(str[5]-'3'));
	else if(str=="COLOR4_UBYTE")
		a = COLOR4_UBYTE;
	else if(str=="TANGENT3")
		a = TANGENT3;
	else if(str=="BINORMAL3")
		a = BINORMAL3;
	else if(str.size()>=9 && !str.compare(0, 8, "TEXCOORD") && str[8]>='1' && str[8]<='4')
	{
		if(str.size()==9)
			a = static_cast<VertexAttribute>(TEXCOORD1+(str[8]-'1'));
		else if(str.size()==11 && str[9]=='_' && str[10]>='0' && str[10]<='3')
			a = make_indexed_attribute(static_cast<VertexAttribute>(TEXCOORD1+(str[8]-'1')), str[10]-'0');
		else
			throw lexical_error(format("conversion of '%s' to VertexAttribute", str));
	}
	else if(str.size()>=10 && !str.compare(0, 6, "GENERIC") && str[7]>='1' && str[7]<='4' && str[8]=='_')
	{
		unsigned n;
		try
		{
			n = lexical_cast<unsigned>(str.substr(9));
		}
		catch(const lexical_error &)
		{
			throw lexical_error(format("conversion of '%s' to VertexAttribute", str));
		}
		a = make_indexed_attribute(static_cast<VertexAttribute>(GENERIC1+(str[7]-'1')), n);
	}
	else
		throw lexical_error(format("conversion of '%s' to VertexAttribute", str));
}

} // namespace GL
} // namespace Msp
